<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Check if class already exists

//if (!class_exists("WPsite_Stats_PF")):

// AJAX functions for impression and click tracking

add_action( 'wp_ajax_wps_pf_record_impression', 		array('WPsite_Stats_PF', 'recordImpresssion'));
add_action( 'wp_ajax_nopriv_wps_pf_record_impression', 	array('WPsite_Stats_PF', 'recordImpresssion'));
add_action( 'wp_ajax_wps_pf_record_click', 				array('WPsite_Stats_PF', 'recordClick'));
add_action( 'wp_ajax_nopriv_wps_pf_record_click', 		array('WPsite_Stats_PF', 'recordClick'));

/**
 * WPsite_Stats_PF class.
 *
 * This class is used to record all stats for popups.
 */
class WPsite_Stats_PF {

	/**
	 * table_name
	 *
	 * (default value: 'wpsite_popup_stats')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $table_name = 'wpsite_popup_stats';

	/**
	 * Create Table when this class is loaded
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
		self::createTable();
	}

	/**
	 * Creates the Stats table if it does not exist
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function createTable(){

		global $wpdb;

		$wpdb->query("
			CREATE TABLE IF NOT EXISTS `" . self::getTableName() . "` (
				`id` int(11) NOT NULL AUTO_INCREMENT,
				`date` date NOT NULL,
				`popup_id` int(11) NOT NULL DEFAULT 0,
				`impressions` int(8) NOT NULL DEFAULT 0,
				`clicks` int(8) NOT NULL DEFAULT 0,
				PRIMARY KEY (`id`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1 AUTO_INCREMENT=1;
		");

	}

	/**
	 * Get the stats for all popups in a timeframe or a single firbar in a timeframe
	 *
	 * @access public
	 * @static
	 * @param mixed $start
	 * @param mixed $end
	 * @param mixed $id (default: null)
	 * @return void
	 */
	static function getStats($start, $end, $id = null, $select = '*') {

		$query = null;

		global $wpdb;

		// All Popups, All Time

		if ($start == null && $end == null && $id == null) {
			$query = 'SELECT ' . $select . ' FROM ' . self::getTableName();
		}

		// All Popups, After Date

		else if ($start != null && $end == null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` = %s', $start);
		}

		// All Popups, Date Range

		else if ($start != null && $end != null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s', $start, $end);
		}

		// Single Popup, All Time

		else if ($start == null && $end == null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `popup_id` = %d', $id);
		}

		// Single Popup, Date Range

		else if ($start != null && $end != null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s AND `popup_id` = %d', $start, $end, $id);
		}

		// No query was created

		if (!isset($query)) {
			return false;
		}

		$result = $wpdb->get_results($query, 'ARRAY_A');

		return $result;
	}

	/**
	 * Runs everyday and will add a record to table if none exists
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function initialize_popup_stats() {

		global $wpdb;
		$today = date(WPSITE_POPUP_DATE_FORMAT);

		include_once(WPSITE_POPUP_PLUGIN_DIR . "admin/model/db.php");

		$popups = $wpdb->get_results("SELECT * FROM `" . WPsite_Popup_DB::getTableName() . "`");

		$result = false;

		foreach ($popups as $popup) {

			// Check if entry already exsits

			$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `popup_id` = %d', $today, $popup->id);
			$popup_impressions = $wpdb->query($query);

			// Entry does not exist, create a new one and set impressions to 0 and clicks to 0

			if (!isset($popup_impressions) || $popup_impressions == 0) {

				$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
					`date`,
					`popup_id`,
					`impressions`,
					`clicks`
					) VALUES (%s, %d, 0, 0)',
					$today,
					$popup->id
				);

				$result = $wpdb->query($query);
			}
		}

		return $result;

	}

	/**
	 * Record an impression made by a popup
	 * This function is called via PHP.
	 *
	 * @access public
	 * @static
	 * @param mixed $data (default: null)
	 * @return void
	 */
	static function recordImpresssion($data = null) {

		$data['popup_id'] = $_POST['popup_id'];

		// Return false if no data is given

		if (!isset($data) || !isset($data['popup_id'])) {
			echo 'Popup impression was NOT able to be recored because no popup id given.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Popup impression was NOT able to be recored because a crawler was detected.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Popup impression was NOT able to be recored because current user is an admin.';
			die();
		}

		global $wpdb;
		$today = date(WPSITE_POPUP_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `popup_id` = %d', $today, $data['popup_id']);
		$popup_impressions = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($popup_impressions) && $popup_impressions != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`impressions` = `impressions` + 1
				WHERE `date` = %s AND `popup_id` = %d', $today, $data['popup_id']);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`popup_id`,
				`impressions`,
				`clicks`
				) VALUES (%s, %d, 1, 0)',
				$today,
				$data['popup_id']
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Popup impression was able to be recored.';
		} else {
			echo 'Popup impression was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls

	}

	/**
	 * Record a click made by a popup
	 * This function is called via AJAX
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function recordClick() {

		$popup_id = $_POST['popup_id'];

		// Return if no data is given

		if (!isset($popup_id)) {
			echo 'Popup click was NOT able to be recored because the Popup ID was not retrieved.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Popup click was NOT able to be recored because current user is not authorized to log clicks.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Popup click was NOT able to be recored because there is no user.  A crawler was detected';
			die();
		}

		global $wpdb;
		$today = date(WPSITE_POPUP_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `popup_id` = %d', $today, $popup_id);
		$popup_clicks = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($popup_clicks) && $popup_clicks != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`clicks` = `clicks` + 1
				WHERE `date` = %s AND `popup_id` = %d', $today, $popup_id);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`popup_id`,
				`impressions`,
				`clicks`
				) VALUES (%s, %d, 0, 1)',
				$today,
				$popup_id
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Popup click was able to be recored.';
		} else {
			echo 'Popup click was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls
	}

	/**
	 * Deletes the stats for a popup when the popup is deleted
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function deleteStats($popup_id) {

		// Return false if popup_id is not set

		if (!isset($popup_id)) {
			return false;
		}

		global $wpdb;

		// Check if entry already exsits

		$query = $wpdb->prepare('DELETE FROM ' . self::getTableName() . ' WHERE `popup_id` = %d', $popup_id);
		$result = $wpdb->query($query);

		return $result;

	}

	/**
	 * Check for BOTs visiting site
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function isBot() {

		if (isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/bot|spider|crawler|curl|^$/i', $_SERVER['HTTP_USER_AGENT'])) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Returns the proper table name based off of site ID
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getTableName(){

		global $wpdb;

		return $wpdb->prefix . self::$table_name;
	}

}

//endif;